==> SimpleMailer-0.80.tar.bz2: SimpleMailer-0.80/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client.Transaction.BuiltIn/IdTransaction.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2011 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

namespace Smdn.Net.Imap4.Client.Transaction.BuiltIn {
  internal sealed class IdTransaction : ImapTransactionBase<ImapCommandResult<IDictionary<string, string>>>, IImapExtension {
    IEnumerable<ImapCapability> IImapExtension.RequiredCapabilities {
      get { yield return ImapCapability.ID; }
    }

    public IdTransaction(ImapConnection connection)
      : base(connection)
    {
    }

    // 3.1. ID Command
    //    Arguments:  client parameter list or NIL
    //    Responses:  OPTIONAL untagged response: ID
    //    Result:     OK    identification information accepted
    //                BAD   command unknown or arguments invalid
    protected override ImapCommand PrepareCommand()
    {
      ImapString clientParameterList;

      if (RequestArguments.TryGetValue("client parameter list", out clientParameterList))
        return Connection.CreateCommand("ID",
                                        clientParameterList);
      else
        return Connection.CreateCommand("ID",
                                        ImapNilString.Nil);
    }

    protected override void OnDataResponseReceived(ImapDataResponse data)
    {
      if (data.Type == ImapDataResponseType.Id)
        serverParameterList = ImapDataResponseConverter.FromId(data);

      base.OnDataResponseReceived(data);
    }

    protected override void OnTaggedStatusResponseReceived(ImapTaggedStatusResponse tagged)
    {
      if (tagged.Condition == ImapResponseCondition.Ok) {
        if (serverParameterList == null)
          serverParameterList = new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase);

        Finish(new ImapCommandResult<IDictionary<string, string>>(serverParameterList.AsReadOnly(),
                                                                  tagged.ResponseText));
      }
      else {
        base.OnTaggedStatusResponseReceived(tagged);
      }
    }

    private IDictionary<string, string> serverParameterList = null;
  }
}

==> SimpleMailer-0.90.tar.bz2: SimpleMailer-0.90/core/Smdn.Net.Imap4.Client/Smdn.Net.Imap4.Client.Transaction.BuiltIn/IdTransaction.cs <==
// 
// Author:
//       smdn <smdn@smdn.jp>
// 
// Copyright (c) 2008-2013 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

using Smdn.Collections;
using Smdn.Net.Imap4.Protocol;
using Smdn.Net.Imap4.Protocol.Client;

#if NET_4_5
using IStringDictionary = System.Collections.Generic.IReadOnlyDictionary<string, string>;
#else
using IStringDictionary = System.Collections.Generic.IDictionary<string, string>;
#endif

namespace Smdn.Net.Imap4.Client.Transaction.BuiltIn {
  internal sealed class IdTransaction : ImapTransactionBase<ImapCommandResult<IStringDictionary>>, IImapExtension {
    IEnumerable<ImapCapability> IImapExtension.RequiredCapabilities {
      get { yield return ImapCapability.ID; }
    }

    public IdTransaction(ImapConnection connection)
      : base(connection)
    {
    }

    // 3.1. ID Command
    //    Arguments:  client parameter list or NIL
    //    Responses:  OPTIONAL untagged response: ID
    //    Result:     OK    identification information accepted
    //                BAD   command unknown or arguments invalid
    protected override ImapCommand PrepareCommand()
    {
      ImapString clientParameterList;

      if (RequestArguments.TryGetValue("client parameter list", out clientParameterList))
        return Connection.CreateCommand("ID",
                                        clientParameterList);
      else
        return Connection.CreateCommand("ID",
                                        ImapNilString.Nil);
    }

    protected override void OnDataResponseReceived(ImapDataResponse data)
    {
      if (data.Type == ImapDataResponseType.Id)
        serverParameterList = ImapDataResponseConverter.FromId(data);

      base.OnDataResponseReceived(data);
    }

    protected override void OnTaggedStatusResponseReceived(ImapTaggedStatusResponse tagged)
    {
      if (tagged.Condition == ImapResponseCondition.Ok) {
        if (serverParameterList == null) {
#if NET_4_5
          serverParameterList = new Dictionary<string, string>(1, StringComparer.OrdinalIgnoreCase); // as empty
#else
          serverParameterList = (new Dictionary<string, string>(1, StringComparer.OrdinalIgnoreCase)).AsReadOnly(); // as empty
#endif
        }

        Finish(new ImapCommandResult<IStringDictionary>(serverParameterList,
                                                        tagged.ResponseText));
      }
      else {
        base.OnTaggedStatusResponseReceived(tagged);
      }
    }

    private IStringDictionary serverParameterList = null;
  }
}
